require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

require "rspec"

describe "ConfigatronPlus" do
  after :each do
    ConfigatronPlus.reset_sources
  end

  describe "#can_use_as_filename?(val)" do
    it "should be true for symbols, strings and Pathnames" do
      ConfigatronPlus.can_use_as_filename?(:testme).should be_true
      ConfigatronPlus.can_use_as_filename?("Hi").should be_true
      ConfigatronPlus.can_use_as_filename?(Pathname("Hi")).should be_true
    end

    it "should be false for anything else" do
      ConfigatronPlus.can_use_as_filename?(123).should be_false
      ConfigatronPlus.can_use_as_filename?({:error => 'yes'}).should be_false
      ConfigatronPlus.can_use_as_filename?([1,2,3]).should be_false
    end
  end

  describe "#valid_list?" do
    it "should be true and errors should be empty for valid @sources" do
      ConfigatronPlus.valid_list?(ConfigatronPlus::DEFAULT_SOURCES).should be_true
      ConfigatronPlus.errors.should == []
    end

    it "should be false and have errors for invalid symbols" do
      ConfigatronPlus.valid_list?([:error]).should be_false
    end
  end

  describe "#sources and #sources=" do
    it "should initialize with defaults" do
      ConfigatronPlus.sources.should == ConfigatronPlus::DEFAULT_SOURCES
    end

    it "should accept a valid sources array" do
      expect {
        ConfigatronPlus.sources = [:current]
        }.should_not raise_error
    end

    it "should not accept an invalid array" do
      expect {ConfigatronPlus.sources = "error"}.should raise_error(ArgumentError)
      expect {ConfigatronPlus.sources = 123 }.should raise_error(ArgumentError)
      expect {ConfigatronPlus.sources = Pathname("/")}.should raise_error(ArgumentError)
      expect {ConfigatronPlus.sources = {:error => "yes indeed"}}.should raise_error(ArgumentError)
      expect {ConfigatronPlus.sources = []}.should_not raise_error(ArgumentError)
      expect {ConfigatronPlus.sources = ["error"]}.should_not raise_error(ArgumentError)
      expect {ConfigatronPlus.sources = [:error]}.should raise_error(ArgumentError)
    end
  end

  describe "#fetch_sources" do
    before :all do
      ConfigatronPlus.stub(:get_from_special)
      ConfigatronPlus.stub(:get_from_file)
    end

    context "default values and all symbols" do
      it "should correctly call #get_from_special and #get_from_file" do
        ConfigatronPlus.should_receive(:get_from_special).exactly(2).times
        ConfigatronPlus.fetch_sources
      end
    end
    context "all strings" do
      it "should correctly call #get_from_special and #get_from_file" do
        ConfigatronPlus.sources = %w(a b c d e)
        ConfigatronPlus.should_receive(:get_from_file).exactly(5).times
        ConfigatronPlus.fetch_sources
      end
    end
    context "mixed strings and symbols" do
      it "should correctly call #get_from_special and #get_from_file" do
        ConfigatronPlus.sources = ['a', :current, 'b', 'c', 'd', 'e', :env]
        ConfigatronPlus.should_receive(:get_from_special).exactly(2).times
        ConfigatronPlus.should_receive(:get_from_file).exactly(5).times
        ConfigatronPlus.fetch_sources
      end
    end
  end

  describe "#is_valid_key_name?(e)" do
    it "should be true for good values" do
      ConfigatronPlus.is_valid_key_name?("a").should be_true
      ConfigatronPlus.is_valid_key_name?("abc").should be_true
      ConfigatronPlus.is_valid_key_name?("ABC").should be_true
      ConfigatronPlus.is_valid_key_name?("abC").should be_true
      ConfigatronPlus.is_valid_key_name?("Cacd").should be_true
      ConfigatronPlus.is_valid_key_name?("a124").should be_true
    end
    it "should be false for bad values" do
      ConfigatronPlus.is_valid_key_name?("1").should be_false
      ConfigatronPlus.is_valid_key_name?("a.1").should be_false
      ConfigatronPlus.is_valid_key_name?("-e").should be_false
      ConfigatronPlus.is_valid_key_name?("po^").should be_false
      ConfigatronPlus.is_valid_key_name?("this is").should be_false
      ConfigatronPlus.is_valid_key_name?("this_is").should be_false
    end
  end

  describe "#use_this_env?(env)" do
    it "should be true when passed a valid name" do
      ConfigatronPlus.use_this_env?('configatron_test').should be_true
      ConfigatronPlus.use_this_env?('configatron_foo_bar').should be_true
      ConfigatronPlus.use_this_env?('configatron_foo_bar_baz').should be_true
    end

    it "should be false when passed an invalid name" do
      ConfigatronPlus.use_this_env?('testing_me').should be_false
      ConfigatronPlus.use_this_env?('t').should be_false
      ConfigatronPlus.use_this_env?('configatron').should be_false
      ConfigatronPlus.use_this_env?('configatron_').should be_false
      ConfigatronPlus.use_this_env?('configatro_foo').should be_false
      ConfigatronPlus.use_this_env?('configatron-bar-baz').should be_false
    end
  end
  describe "#is_symbol_defined?(sym)" do
    it "should be true when passed an existing symbol" do
      ConfigatronPlus.is_symbol_defined?(:env).should be_true
      ConfigatronPlus.is_symbol_defined?(:current).should be_true
      ConfigatronPlus.config.source_files[:testing] = "test"
      ConfigatronPlus.is_symbol_defined?(:testing).should be_true
    end

    it "should be false when passed any other symbol or object" do
      ConfigatronPlus.is_symbol_defined?(:foo).should be_false
      ConfigatronPlus.is_symbol_defined?('foo').should be_false
      ConfigatronPlus.is_symbol_defined?(123).should be_false
      ConfigatronPlus.is_symbol_defined?([1,2,3]).should be_false
    end
  end
  describe "#get_from_env" do
    it "should set configatron settings from environment" do
      ENV['configatron_test_one'] = "123"
      ENV['configatron_test_two'] = "abc"
      ENV['configatron_another'] = "foo,bar,baz"

      ConfigatronPlus.get_from_env
      configatron.test.one.should == "123"
      configatron.test.two.should == "abc"
      configatron.another.should == "foo,bar,baz"
    end
  end

  describe "#get_from_special(s)" do
    before :each do
      ConfigatronPlus.stub(:get_from_file)
    end
    it "should not raise an error for the default settings (less env)" do
      [:current, :env].each do |sym|
        expect {
          ConfigatronPlus.get_from_special(sym)
        }.should_not raise_error ConfigatronPlus::ConfigError
      end
    end

    it "should raise an error if setting does not exist for symbol" do
      expect {
        ConfigatronPlus.get_from_special(:bad)
      }.should raise_error ConfigatronPlus::ConfigError
    end

    it "should call get_from_file when called correctly" do
      ConfigatronPlus.should_receive(:get_from_file).once
      ConfigatronPlus.get_from_special(:current)
    end
  end

  describe "#get_from_file(file)" do
    after :each do
      ConfigatronPlus.reset_sources
      ConfigatronPlus.fail_when_missing = false
    end

    it "should raise an error if not called with a pathname" do
      expect {
        ConfigatronPlus.get_from_file("failure")
      }.should raise_error ArgumentError
    end

    context "fail_when_missing is true" do
      it "should raise an error when the file doesn't exist" do
        ConfigatronPlus.fail_when_missing = true
        expect {
          ConfigatronPlus.get_from_file(Pathname.new("/some/long/foo/bar/baz/path"))
        }.should raise_error ConfigatronPlus::ConfigError
      end
    end
    context "fail_when_missing is false" do
      it "should not raise an error when the file doesn't exist" do
        ConfigatronPlus.fail_when_missing = false
        expect {
          ConfigatronPlus.get_from_file(Pathname.new("/some/long/foo/bar/baz/path"))
        }.should_not raise_error ConfigatronPlus::ConfigError

      end
    end

    it "should read a file into configatron when file is present" do
      tmp = Pathname('spec/config_files/testing_config.rb')
      ConfigatronPlus.get_from_file(tmp)
      configatron.file.test.one.should == 123
      configatron.file.test.two.should == "abc"
      configatron.file.another.should == 'xyz'
      configatron.file.foo.bar.baz.should == 'woot'
    end
  end
end